/* This file was generated by syncqt. */
#ifndef QT_QTQUICKTEST_VERSION_H
#define QT_QTQUICKTEST_VERSION_H

#define QTQUICKTEST_VERSION_STR "5.9.2"

#define QTQUICKTEST_VERSION 0x050902

#endif // QT_QTQUICKTEST_VERSION_H
